@@ -32,7 +32,6 @@ import("utils.progress")
 import("private.cache.build_cache")
 import("private.service.distcc_build.client", {alias = "distcc_build_client"})
 
--- init it
 function init(self)
 
     -- init mxflags
@@ -44,16 +43,6 @@ function init(self)
     -- init shflags
     self:set("shflags", "-shared")
 
-    -- add -fPIC for shared
-    --
-    -- we need check it for clang/gcc with window target
-    -- @see https://github.com/xmake-io/xmake/issues/1392
-    --
-    if not self:is_plat("windows", "mingw") and self:has_flags("-fPIC", "cxflags") then
-        self:add("shflags", "-fPIC")
-        self:add("shared.cxflags", "-fPIC")
-    end
-
     -- init flags map
     self:set("mapflags", {
         -- warnings
@@ -76,6 +65,21 @@ function init(self)
     end
 end
 
+-- we can only call has_flags in load(),
+-- as it requires the full platform toolchain flags.
+--
+function load(self)
+    -- add -fPIC for shared
+    --
+    -- we need check it for clang/gcc with window target
+    -- @see https://github.com/xmake-io/xmake/issues/1392
+    --
+    if not self:is_plat("windows", "mingw") and self:has_flags("-fPIC") then
+        self:add("shflags", "-fPIC")
+        self:add("shared.cxflags", "-fPIC")
+    end
+end
+
 -- make the strip flag
 function nf_strip(self, level, target)
     local maps = {
@@ -83,7 +87,10 @@ function nf_strip(self, level, target)
     ,   all   = "-s"
     }
     if self:is_plat("macosx", "iphoneos") then
-        maps.all = "-Wl,-x"
+        maps.all = {"-Wl,-x", "-Wl,-dead_strip"}
+    elseif self:is_plat("windows") then
+        -- clang does not it on windows, TODO maybe we need test it for gcc
+        maps = {}
     end
     return maps[level]
 end
@@ -166,6 +173,7 @@ function nf_vectorext(self, extension)
     ,   ssse3 = "-mssse3"
     ,   avx   = "-mavx"
     ,   avx2  = "-mavx2"
+    ,   fma   = "-mfma"
     ,   neon  = "-mfpu=neon"
     }
     return maps[extension]
@@ -181,6 +189,8 @@ function nf_language(self, stdname)
             ansi        = "-ansi"
         ,   c89         = "-std=c89"
         ,   gnu89       = "-std=gnu89"
+        ,   c90         = "-std=c90"
+        ,   gnu90       = "-std=gnu90"
         ,   c99         = "-std=c99"
         ,   gnu99       = "-std=gnu99"
         ,   c11         = "-std=c11"
@@ -246,7 +256,7 @@ end
 
 -- make the define flag
 function nf_define(self, macro)
-    return "-D" .. macro
+    return {"-D" .. macro}
 end
 
 -- make the undefine flag
@@ -397,7 +407,7 @@ function link(self, objectfiles, targetkind, targetfile, flags)
     if option.get("verbose") then
         os.execv(program, argv, {envs = self:runenvs()})
     else
-        os.runv(program, argv, {envs = self:runenvs()})
+        os.vrunv(program, argv, {envs = self:runenvs()})
     end
 end
 
@@ -440,6 +450,8 @@ function _preprocess(program, argv, opt)
             is_gcc = true
         elseif tool:name():startswith("clang") then
             is_clang = true
+        elseif tool:name() == "circle" then
+            return
         end
     end
 
